//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX
#include "progs/globals_vertex.inc"

uniform vec4 uRTSize;

#define SMAA_GLSL_3
#define SMAA_RT_METRICS uRTSize
#define SMAA_PRESET_ULTRA
#define SMAA_INCLUDE_VS 1
#define SMAA_INCLUDE_PS 0

#include "progs/SMAA/SMAA.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(0, vec3);
    var_attrib(1, vec2);
    var_attrib(2, vec4);
    var_attrib(3, int);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord,     TEXCOORD0)
    def_var_out(vec4, out_smaaCoord1,   TEXCOORD1)
    def_var_out(vec4, out_smaaCoord2,   TEXCOORD2)
    def_var_out(vec4, out_smaaCoord3,   TEXCOORD3)
    def_var_out(vec4, out_smaaCoord4,   TEXCOORD4)
    def_var_out(vec4, out_point,        POINT)
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    outVarPosition(output, position)    = mul(uProjectionMatrix, mul(uModelViewMatrix, vertex));
    outVar(output, out_texcoord)        = inVarAttrib(1, input);
    outVar(output, out_point)           = uFrustumCorners[inVarAttrib(3, input)];
    
    vec4 vOffset[3];
    vec2 vFragCoord;
    SMAABlendingWeightCalculationVS(inVarAttrib(1, input), vFragCoord, vOffset);
    
    outVar(output, out_smaaCoord1) = vOffset[0];
    outVar(output, out_smaaCoord2) = vOffset[1];
    outVar(output, out_smaaCoord3) = vOffset[2];
    outVar(output, out_smaaCoord4) = vec4(vFragCoord, 0.0, 0.0);
    
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/globals_fragment.inc"

uniform vec4 uRTSize;

#define SMAA_GLSL_3
#define SMAA_RT_METRICS uRTSize
#define SMAA_PRESET_ULTRA
#define SMAA_INCLUDE_VS 0
#define SMAA_INCLUDE_PS 1

#include "progs/SMAA/SMAA.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord,      TEXCOORD0)
    def_var_in(vec4, out_smaaCoord1,    TEXCOORD1)
    def_var_in(vec4, out_smaaCoord2,    TEXCOORD2)
    def_var_in(vec4, out_smaaCoord3,    TEXCOORD3)
    def_var_in(vec4, out_smaaCoord4,    TEXCOORD4)
    def_var_in(vec4, out_point,         POINT)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

//----------------------------------------------------
shader_main(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec4 vOffset[3];
    vOffset[0] = out_smaaCoord1;
    vOffset[1] = out_smaaCoord2;
    vOffset[2] = out_smaaCoord3;
    
    vec2 vFragCoord = out_smaaCoord4.xy;
    vec4 vSubSamples = vec4(0.0);
    
    vec4 vColor = SMAABlendingWeightCalculationPS(
        out_texcoord,
        vFragCoord,
        vOffset,
        uTex0, // edge render target
        uTex1, // area
        uTex2, // search
        vSubSamples);
    
    outVarFragment(output, fragment) = vColor;
    outReturn(output)
}

#endif
